@@ -4,15 +4,15 @@ use std::str::{self};
 #[derive(Clone, Debug, PartialEq, PartialOrd)]
 pub enum ArgTypes {
 	// should always be first to set a global variable before executing related functions
-	DeviceName = isize::MIN,
-	TopMargin = isize::MIN + 1,
-	MaxVolume = isize::MIN + 2,
-	CustomIcon = isize::MIN + 3,
-	Player = isize::MIN + 4,
-	MonitorName = isize::MIN + 5,
-	CustomProgressText = isize::MIN + 6,
+	DeviceName = (i32::MIN) as isize,
+	TopMargin = (i32::MIN + 1) as isize,
+	MaxVolume = (i32::MIN + 2) as isize,
+	CustomIcon = (i32::MIN + 3) as isize,
+	Player = (i32::MIN + 4) as isize,
+	MonitorName = (i32::MIN + 5) as isize,
+	CustomProgressText = (i32::MIN + 6) as isize,
+	MinBrightness = (i32::MIN + 7) as isize,
 	// Other
-	None = 0,
 	CapsLock = 1,
 	SinkVolumeRaise = 2,
 	SinkVolumeLower = 3,
@@ -28,12 +28,13 @@ pub enum ArgTypes {
 	CustomMessage = 13,
 	Playerctl = 14,
 	CustomProgress = 15,
+	CustomSegmentedProgress = 16,
+	KbdBacklight = 17,
 }
 
 impl fmt::Display for ArgTypes {
 	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
 		let string = match self {
-			ArgTypes::None => "NONE",
 			ArgTypes::CapsLock => "CAPSLOCK",
 			ArgTypes::MaxVolume => "MAX-VOLUME",
 			ArgTypes::SinkVolumeRaise => "SINK-VOLUME-RAISE",
@@ -55,9 +56,12 @@ impl fmt::Display for ArgTypes {
 			ArgTypes::Player => "PLAYER",
 			ArgTypes::MonitorName => "MONITOR-NAME",
 			ArgTypes::CustomProgress => "CUSTOM-PROGRESS",
+			ArgTypes::CustomSegmentedProgress => "CUSTOM-SEGMENTED-PROGRESS",
 			ArgTypes::CustomProgressText => "CUSTOM-PROGRESS-TEXT",
+			ArgTypes::MinBrightness => "MIN-BRIGHTNESS",
+			ArgTypes::KbdBacklight => "KBD-BACKLIGHT",
 		};
-		return write!(f, "{}", string);
+		write!(f, "{}", string)
 	}
 }
 
@@ -87,7 +91,10 @@ impl str::FromStr for ArgTypes {
 			"PLAYER" => ArgTypes::Player,
 			"MONITOR-NAME" => ArgTypes::MonitorName,
 			"CUSTOM-PROGRESS" => ArgTypes::CustomProgress,
+			"CUSTOM-SEGMENTED-PROGRESS" => ArgTypes::CustomSegmentedProgress,
 			"CUSTOM-PROGRESS-TEXT" => ArgTypes::CustomProgressText,
+			"MIN-BRIGHTNESS" => ArgTypes::MinBrightness,
+			"KBD-BACKLIGHT" => ArgTypes::KbdBacklight,
 			other_type => return Err(other_type.to_owned()),
 		};
 		Ok(result)
