@@ -1,14 +1,15 @@
-use zbus::{dbus_interface, Connection, ConnectionBuilder, SignalContext};
+use zbus::object_server::SignalEmitter;
+use zbus::{connection, interface, Connection};
 
 use crate::config::{DBUS_BACKEND_NAME, DBUS_PATH};
 
 pub struct DbusServer;
 
-#[dbus_interface(name = "org.erikreider.swayosd")]
+#[interface(name = "org.erikreider.swayosd")]
 impl DbusServer {
-	#[dbus_interface(signal)]
+	#[zbus(signal)]
 	pub async fn key_pressed(
-		signal_ctxt: &SignalContext<'_>,
+		signal_ctxt: &SignalEmitter<'_>,
 		key_code: u16,
 		state: i32,
 	) -> zbus::Result<()>;
@@ -16,7 +17,7 @@ impl DbusServer {
 
 impl DbusServer {
 	async fn get_connection(&self) -> zbus::Result<Connection> {
-		let conn = ConnectionBuilder::system()?
+		let conn = connection::Builder::system()?
 			.name(DBUS_BACKEND_NAME)?
 			.serve_at(DBUS_PATH, DbusServer)?
 			.build()
