@@ -9,6 +9,7 @@ use gtk::{
 };
 use pulsectl::controllers::types::DeviceInfo;
 
+use crate::widgets::segmented_progress_widget::SegmentedProgressWidget;
 use crate::{
 	brightness_backend::BrightnessBackend,
 	utils::{
@@ -30,6 +31,9 @@ pub struct SwayosdWindow {
 	timeout_id: Rc<RefCell<Option<glib::SourceId>>>,
 }
 
+// TODO: Use custom widget
+// - Use start, center, and end children
+//   - Always center the centered widget (both left and right sides are the same width)
 impl SwayosdWindow {
 	/// Create a new window and assign it to the given application.
 	pub fn new(app: &gtk::Application, monitor: &gdk::Monitor) -> Self {
@@ -43,7 +47,8 @@ impl SwayosdWindow {
 
 		window.set_exclusive_zone(-1);
 		window.set_layer(gtk_layer_shell::Layer::Overlay);
-		window.set_anchor(gtk_layer_shell::Edge::Top, true);
+		// Anchor to bottom edge for better reliability with rotated/transformed displays
+		window.set_anchor(gtk_layer_shell::Edge::Bottom, true);
 
 		// Set up the widgets
 		window.set_width_request(250);
@@ -66,12 +71,13 @@ impl SwayosdWindow {
 		let update_margins = |window: &gtk::ApplicationWindow, monitor: &gdk::Monitor| {
 			// Monitor scale factor is not always correct
 			// Transform monitor height into coordinate system of window
-			let mon_height =
-				monitor.geometry().height() * monitor.scale_factor() / window.scale_factor();
-			// Calculate new margin
-			let bottom = mon_height - window.allocated_height();
-			let margin = (bottom as f32 * get_top_margin()).round() as i32;
-			window.set_margin(gtk_layer_shell::Edge::Top, margin);
+			let mon_height = monitor.geometry().height() / window.scale_factor();
+			// Calculate margin from bottom while preserving top_margin semantics:
+			// top_margin=0.85 means window should be at 85% from top, which equals
+			// 15% from bottom. By anchoring to bottom, we avoid issues with
+			// window.allocated_height() being 0 or incorrect during initialization.
+			let margin = (mon_height as f32 * (1.0 - get_top_margin())).round() as i32;
+			window.set_margin(gtk_layer_shell::Edge::Bottom, margin);
 		};
 
 		// Set the window margin
@@ -115,7 +121,7 @@ impl SwayosdWindow {
 		};
 		let icon_state = &match (device.mute, volume) {
 			(true, _) => "muted",
-			(_, x) if x == 0.0 => "muted",
+			(_, 0.0) => "muted",
 			(false, x) if x > 0.0 && x <= 33.0 => "low",
 			(false, x) if x > 33.0 && x <= 66.0 => "medium",
 			(false, x) if x > 66.0 && x <= 100.0 => "high",
@@ -131,7 +137,7 @@ impl SwayosdWindow {
 
 		let icon = self.build_icon_widget(icon_name);
 		let progress = self.build_progress_widget(volume / max_volume);
-		let label = self.build_text_widget(Some(&format!("{}%", volume)));
+		let label = self.build_text_widget(Some(&format!("{}%", volume)), Some(4));
 
 		progress.set_sensitive(!device.mute);
 
@@ -153,7 +159,10 @@ impl SwayosdWindow {
 		let brightness = brightness_backend.get_current() as f64;
 		let max = brightness_backend.get_max() as f64;
 		let progress = self.build_progress_widget(brightness / max);
-		let label = self.build_text_widget(Some(&format!("{}%", (brightness / max * 100.) as i32)));
+		let label = self.build_text_widget(
+			Some(&format!("{}%", (brightness / max * 100.).round() as i32)),
+			Some(4),
+		);
 
 		self.container.append(&icon);
 		self.container.append(&progress);
@@ -168,7 +177,8 @@ impl SwayosdWindow {
 		self.clear_osd();
 
 		let icon = self.build_icon_widget(icon);
-		let label = self.build_text_widget(label);
+		let label = self.build_text_widget(label, None);
+		label.set_hexpand(true);
 
 		self.container.append(&icon);
 		self.container.append(&label);
@@ -176,10 +186,36 @@ impl SwayosdWindow {
 		self.run_timeout();
 	}
 
+	pub fn changed_kbd_backlight(&self, value: u32, max: u32) {
+		self.clear_osd();
+
+		let value = value.min(max);
+
+		let icon_name = match value {
+			0 => "keyboard-brightness-off-symbolic",
+			v if (v == max) => "keyboard-brightness-high-symbolic",
+			_ => "keyboard-brightness-medium-symbolic",
+		};
+		let icon = self.build_icon_widget(icon_name);
+		self.container.append(&icon);
+
+		// A segmented progress bar looks cramped when there are too many segments
+		if max < 5 {
+			let progress = self.build_segmented_progress_widget(value, max);
+			self.container.append(&progress);
+		} else {
+			let progress = self.build_progress_widget((value / max) as f64);
+			self.container.append(&progress);
+		}
+
+		self.run_timeout();
+	}
+
 	pub fn changed_keylock(&self, key: KeysLocks, state: bool) {
 		self.clear_osd();
 
-		let label = self.build_text_widget(None);
+		let label = self.build_text_widget(None, None);
+		label.set_hexpand(true);
 
 		let on_off_text = match state {
 			true => "On",
@@ -227,7 +263,33 @@ impl SwayosdWindow {
 		self.container.append(&progress);
 
 		if let Some(text) = text {
-			let label = self.build_text_widget(Some(text.deref()));
+			let label = self.build_text_widget(Some(text.deref()), None);
+			self.container.append(&label);
+		}
+
+		self.run_timeout();
+	}
+
+	pub fn custom_segmented_progress(
+		&self,
+		value: u32,
+		n_segments: u32,
+		text: Option<String>,
+		icon_name: Option<&str>,
+	) {
+		self.clear_osd();
+
+		if let Some(icon_name) = icon_name {
+			let icon = self.build_icon_widget(icon_name);
+			self.container.append(&icon);
+		}
+
+		let value = value.min(n_segments);
+		let progress = self.build_segmented_progress_widget(value, n_segments);
+		self.container.append(&progress);
+
+		if let Some(text) = text {
+			let label = self.build_text_widget(Some(text.deref()), None);
 			self.container.append(&label);
 		}
 
@@ -237,7 +299,8 @@ impl SwayosdWindow {
 	pub fn custom_message(&self, message: &str, icon_name: Option<&str>) {
 		self.clear_osd();
 
-		let label = self.build_text_widget(Some(message));
+		let label = self.build_text_widget(Some(message), None);
+		label.set_hexpand(true);
 
 		if let Some(icon_name) = icon_name {
 			let icon = self.build_icon_widget(icon_name);
@@ -294,11 +357,13 @@ impl SwayosdWindow {
 		}
 	}
 
-	fn build_text_widget(&self, text: Option<&str>) -> gtk::Label {
+	fn build_text_widget(&self, text: Option<&str>, min_chars: Option<u32>) -> gtk::Label {
 		cascade! {
 			gtk::Label::new(text);
+			// width-chars is based off of the average font width, so we add 1
+			// to make sure that it's wide enough.
+			..set_width_chars(min_chars.map_or(-1, |v| (v + 1) as i32));
 			..set_halign(gtk::Align::Center);
-			..set_hexpand(true);
 			..add_css_class("title-4");
 		}
 	}
@@ -311,4 +376,17 @@ impl SwayosdWindow {
 			..set_hexpand(true);
 		}
 	}
+
+	fn build_segmented_progress_widget(
+		&self,
+		value: u32,
+		n_segments: u32,
+	) -> SegmentedProgressWidget {
+		cascade! {
+			SegmentedProgressWidget::new(n_segments);
+			..set_value(value);
+			..set_valign(gtk::Align::Center);
+			..set_hexpand(true);
+		}
+	}
 }
